import { Injectable } from '@angular/core';
import { Post } from '../../../model/Post';
import { AuthService } from '../auth/auth.service';
import { AngularFirestore, Reference } from '@angular/fire/compat/firestore';
import { User } from '../../../model/User';
import { UserService } from './user.service';
import { DocumentReference } from '@angular/fire/firestore';

@Injectable({
  providedIn: 'root'
})
export class PostService {
  constructor(private auth: AuthService, private afs: AngularFirestore, private usersvc: UserService) { }

  getFeed(length: Number, start: Number): Promise<Array<Post>> {
    return new Promise<Array<Post>>((resolve, reject) => {
      let result = Array<Post>();

      if (this.auth.checkAllowed()) {
        this.usersvc.getCurrentUser()
          .then(currentUser => {
            // get posts
            this.afs.collection<Post>('post').valueChanges({ idField: 'id' }).subscribe(posts => {
              posts.forEach(post => {
                // get the user who made this post
                this.afs.doc((post as any).user).valueChanges().subscribe(user => {
                  // get how many likes it has
                  this.afs.collection('like', ref => ref.where('post', '==', this.afs.doc('/post/' + post.id).ref)).valueChanges().subscribe(likes => {
                    // check if we liked this post
                    let liked: boolean = likes.filter(like => (like as any).user.id == currentUser.username).length != 0;
                    result.push({
                      user: {
                        username: (user as User).username,
                        email: (user as User).email,
                        picture: (user as User).picture,
                        followed: (user as User).followed
                      },
                      content: post.content,
                      image: post.image,
                      likes: likes.length,
                      liked: liked,
                      id: post.id
                    })
                  })
                });
              });
              resolve(result);
            });
          })
      } else {
        reject()
      }
    });
  }

  getPosts(username: String, lenght: Number, start: Number) {
    return new Promise<Array<Post>>((resolve, reject) => {
      if (this.auth.checkAllowed()) {
        if (username == undefined) {
          this.auth.getUser().subscribe(cred => {
            // get the user by id
            this.afs.collection<User>('user', ref => ref.where('id', '==', cred?.uid)).valueChanges().subscribe(user => {
              // get their posts
              this.afs.collection<Post>('post', ref => ref.where('user', '==', this.afs.doc('/user/' + user[0].username).ref)).valueChanges({ idField: 'id' }).subscribe(posts => {
                posts.forEach(post => {
                  // get how many likes it has
                  this.afs.collection('like', ref => ref.where('post', '==', this.afs.doc('/post/' + post.id).ref)).valueChanges().subscribe(likes => {
                    // check if we liked this post
                    post.liked = likes.filter(like => (like as any).user.id == user[0].username).length != 0;
                    post.user = user[0] as User;
                    post.likes = likes.length;
                  })
                  resolve(posts)
                })
              });
            })
          });
        } else {
          this.usersvc.getCurrentUser()
            .then(currentUser => {
              // get the user by username
              this.afs.collection<User>('user').doc(username as string).valueChanges().subscribe(user => {
                // get their posts
                this.afs.collection<Post>('post', ref => ref.where('user', '==', this.afs.doc('/user/' + username).ref)).valueChanges({ idField: 'id' }).subscribe(posts => {
                  posts.forEach(post => {
                    // get how many likes it has
                    this.afs.collection('like', ref => ref.where('post', '==', this.afs.doc('/post/' + post.id).ref)).valueChanges().subscribe(likes => {
                      post.liked = likes.filter(like => (like as any).user.id == currentUser.username).length != 0;
                      post.user = user as User;
                      post.likes = likes.length;
                    })
                    resolve(posts)
                  })
                });
              });
            })
        }
      } else {
        reject()
      }
    });
  }

  // FIX: duplicate posts
  newPost(content: String, image: String) {
    return new Promise<void>((resolve, reject) => {
      console.log('new post')
      if (this.auth.checkAllowed()) {
        this.auth.getUser().subscribe(data => {
          this.afs.collection<User>('user', ref => ref.where('id', '==', data?.uid)).valueChanges().subscribe(user => {
            this.afs.collection('post').add({
              user: this.afs.doc('/user/' + user[0].username).ref,
              content: content,
              image: image,
            })
              .then(() => resolve())
              .catch(() => reject());
          });
        });
      } else {
        reject();
      }
    });
  }

  removePost(id: String) {
    return this.afs.doc('/post/' + id).delete();
  }

  toggleLike(postId: String): Promise<boolean> {
    return new Promise<boolean>((resolve, reject) => {
      if (this.auth.checkAllowed()) {
        this.usersvc.getCurrentUser()
          .then(user => {
            let userRef = this.afs.doc('/user/' + user.username).ref;
            let postRef = this.afs.doc('/post/' + postId).ref;
            this.afs.collection('like', ref => ref.where('user', '==', userRef).where('post', '==', postRef)).get().subscribe(likes => {
              if (likes.docs.length == 0) {
                // like
                this.afs.collection('like').add({
                  user: userRef,
                  post: postRef
                })
                resolve(true);
              } else {
                // dislike
                likes.docs.forEach(likes => {
                  likes.ref.delete()
                })
                resolve(false);
              }
            })
          })
          .catch(() => reject())
      } else {
        reject();
      }
    });
  }
}
